/******************************
 * Hereta de Controlador
 * ****************************/

using System;
using System.Collections;
using System.Text;
using NHibernate;
using NHibernate.Cfg;
using System.Windows.Forms;
using NHibernate.Expression;
using NHibernate.Id;


namespace PFC.Controller
{
   public  class ControlComanda : Controlador
    {
       static private ControlComanda _instance;

      
       
       //*********************************Obtencio de totes les comandes de la BD*********************************      
       public IList loadComandes()
       {
           IList comandes = null;

           try
           {
               ReconnectSession(); 
               comandes = session.CreateCriteria(typeof(Comanda)).List();

           }
           catch (HibernateException ex)
           {

               throw new Exception("ControlComanda.cs:Error Carregant comandes: " + ex.ToString());

           }

           return comandes;
       }


       //*********************************Obtenci de com segons el seu Id*********************************
       public Comanda loadComanda(int id)
       {
           Comanda comanda = null;

           try
           {
               comanda = (Comanda)session.Load(typeof(Comanda), id);

               session.Refresh(comanda);
           }
           catch (HibernateException ex)
           {
               throw new Exception("ControlComanda.cs:No s'ha pogut carregar la comanda " + id + ":" + ex.Message);
           }

           return comanda;
       }

       //*********************************Obtencio de la comanda Actual en la taula passada per parmetre*********************************      
       public Comanda loadComandaActual(Taula t)
       {
           IList comandes = null;
           Comanda comandaActual=null;

           try
           {

               comandes = session.Find("from PFC.Comanda as c where c.Taula.Id=" + t.Id + " and c.Estat not like 'Cobrada' and c.Estat not like 'No cobrada'");
              foreach (Comanda c in comandes)
                  comandaActual = c;
              session.Refresh(comandaActual);
              
           }
           catch (HibernateException ex)
           {

               throw new Exception("ControlComanda.cs:Error Carregant comanda actual: " + ex.ToString());

           }

           return comandaActual;
       }



       //*********************************Obtencio de tots els comandes pendents de la BD*********************************      
       public IList loadCobrosPendents()
       {

           IList i = null;

           try
           {

               i = session.Find("from PFC.Comanda as c where c.Estat like 'No cobrada'");

           }
           catch (HibernateException ex)
           {
               throw new Exception("ControlComanda.cs:No s'han pogut carregar les comandes " + ex.Message);
           }

           return i;

       }



       //*********************************Guarda a la BD la comanda passada per parmetre*********************************      
       public bool saveComanda(Comanda c)
       {
           ITransaction tx = null;
               bool res = true;
               ReconnectSession();
               try
               {
                  
                   tx = session.BeginTransaction();
                   c.Data = System.DateTime.Now;
                   if (c.Id == 0) //Valor de unsaved-value !!Abans d'estar guardat t el 0
                   {

                       c.Estat = "Inicial";
                       session.Save(c);

                   }
                   else
                   {
                       session.Update(c);
                   }
                   tx.Commit();


                   
               }
               catch (HibernateException ex)
               {
                   tx.Rollback();
                   res = false;
                   throw new Exception("ControlComanda.cs:No s'ha pogut guardar la comanda: " + ex.ToString());
                   
               }
               
               return res;
           }


           //*********************************Esborra de la BD la lineaComanda*********************************    
           public bool deleteComanda(Comanda c)
           {
               bool res = true;
               ITransaction tx = null;
               try
               {

                   tx = session.BeginTransaction();
                   session.Delete(c);
                   tx.Commit();

               }
               catch (HibernateException ex)
               {
                   tx.Rollback();
                   res = false;
                   session.Clear(); //Neteja els possibles save,update i delete pendents per causa d'algun error
                   throw new Exception("ControlComanda.cs:No s'ha pogut esborrar la Comanda " + ex.ToString());

               }

               return res;
           }


       //*********************************Patr singleton:Obtenci sempre de la mateixa instancia*********************************      
       public static ControlComanda getInstance()
       {
           if (_instance == null) { _instance = new ControlComanda(); }
           return _instance;
       }

       }
     
       
    }


